import logging
import asyncio
import time

from .http_client import HTTP
from .gateway import WebSocket
from .utils import exceptions
from .channel import DiscordChannel
from .utils import Logging

log = logging.getLogger(__name__)
log.setLevel(Logging.client.LEVEL)
logging.basicConfig(format=Logging.FORMAT)

class Client:
    def __init__(self, token, prefix=""):
        self.token = token
        self.prefix = prefix
        self.alive = True
        
        self.session_id = ""
        self.username   = ""
        self.id         = 0
        self.app_id     = 0
        self.guilds     = []
        self.avatar     = ""

    async def run_loop(self):
        while self.alive:
            if self.ws.heartbeat_interval != 0:
                current_time = round(time.time()*1000)
                if (current_time - self.ws.last_heartbeat) > (self.ws.heartbeat_interval):
                    await self.ws.send_heartbeat()
                    self.ws.last_heartbeat = current_time
            
            await self.ws.event_loop()

    def run(self):
        self.loop = asyncio.get_event_loop()
        self.loop.set_exception_handler(exceptions.exception_handler)

        self.ws = WebSocket(token=self.token, loop=self.loop)
        self.handle_events()

        self.loop.create_task(self.run_loop())

        try:
            self.loop.run_forever()
        except KeyboardInterrupt:
            self.stop()
        except Exception as error:
            log.error(error)

    def stop(self):
        if self.ws != None and not self.ws.socket.closed:
            self.loop.create_task(self.ws.close())

    async def _on_ready(self, data):
        self.username   = data["d"]["user"]["username"]
        self.client_id  = data["d"]["user"]["id"]
        self.avatar     = data["d"]["user"]["avatar"]
        self.guilds     = data["d"]["guilds"]
        self.app_id     = data["d"]["application"]["id"]
        self.session_id = data["d"]["session_id"]
        await self.on_ready()

    def handle_events(self):
        self.ws.events.on_ready = self._on_ready

    def Channel(self, channel_id):
        channel = DiscordChannel(channel_id, self.token, self.loop)
        return channel

    async def send(self, message, channel=0, tts=False):
        await self.send_message(message, channel, tts)

    async def send_message(self, content, channel_id, tts):
        data = {
            "content": message,
            "tts": tts
        }
        await self.http.post("/channels/" + str(channel) + "/messages", data=data)


