import logging
import json
import asyncio
import aiohttp
from .utils import Logging
from .utils import API

log = logging.getLogger(__name__)
log.setLevel(Logging.http_client.LEVEL)
logging.basicConfig(format=Logging.FORMAT)

class HTTP:
    BASE = f"https://discordapp.com/api/v{API.LEVEL}"

    def __init__(self, token, loop=None):
        self.loop = asyncio.get_event_loop() if loop is None else loop
        self.token = token

    async def get(self, url):
        headers = {"Authorization": "Bot " + self.token, "Content-Type": "application/json"}

        log.info("GET " + self.BASE + url)

        self.create_session()
        response = await self._session.get(self.BASE + url, headers=headers)
        await self.destroy_session()

        result = await response.text()
        log.debug("Result: " + result)
        return result


    async def post(self, url, data):
        headers = {"Authorization": "Bot " + self.token, "Content-Type": "application/json"}
        data = json.dumps(data)

        log.info("POST " + self.BASE + url)
        log.debug("Payload: " + json.dumps(data, indent=2))

        self.create_session()
        response = await self._session.post(self.BASE + url, data=data, headers=headers)
        await self.destroy_session()

        result = await response.text()
        log.debug("Result: " + result)
        return result

    async def delete(self, url):
        headers = {"Authorization": "Bot " + self.token, "Content-Type": "application/json"}

        log.info("DELETE " + self.BASE + url)

        self.create_session()
        response = await self._session.post(self.BASE + url, headers=headers)
        await self.destroy_session()

        result = await response.text()
        log.debug("Result: " + result)
        return result


    def create_session(self):
        self._session = aiohttp.ClientSession()

    async def destroy_session(self):
        await self._session.close()